"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MfaService = void 0;
const uuid_1 = require("uuid");
const typedi_1 = require("typedi");
const n8n_core_1 = require("n8n-core");
const user_repository_1 = require("../databases/repositories/user.repository");
const totp_service_1 = require("./totp.service");
let MfaService = class MfaService {
    constructor(userRepository, totp, cipher) {
        this.userRepository = userRepository;
        this.totp = totp;
        this.cipher = cipher;
    }
    generateRecoveryCodes(n = 10) {
        return Array.from(Array(n)).map(() => (0, uuid_1.v4)());
    }
    generateEncryptedRecoveryCodes() {
        return this.generateRecoveryCodes().map((code) => this.cipher.encrypt(code));
    }
    async saveSecretAndRecoveryCodes(userId, secret, recoveryCodes) {
        const { encryptedSecret, encryptedRecoveryCodes } = this.encryptSecretAndRecoveryCodes(secret, recoveryCodes);
        return await this.userRepository.update(userId, {
            mfaSecret: encryptedSecret,
            mfaRecoveryCodes: encryptedRecoveryCodes,
        });
    }
    encryptSecretAndRecoveryCodes(rawSecret, rawRecoveryCodes) {
        const encryptedSecret = this.cipher.encrypt(rawSecret), encryptedRecoveryCodes = rawRecoveryCodes.map((code) => this.cipher.encrypt(code));
        return {
            encryptedRecoveryCodes,
            encryptedSecret,
        };
    }
    decryptSecretAndRecoveryCodes(mfaSecret, mfaRecoveryCodes) {
        return {
            decryptedSecret: this.cipher.decrypt(mfaSecret),
            decryptedRecoveryCodes: mfaRecoveryCodes.map((code) => this.cipher.decrypt(code)),
        };
    }
    async getSecretAndRecoveryCodes(userId) {
        const { mfaSecret, mfaRecoveryCodes } = await this.userRepository.findOneOrFail({
            where: { id: userId },
            select: ['id', 'mfaSecret', 'mfaRecoveryCodes'],
        });
        return this.decryptSecretAndRecoveryCodes(mfaSecret !== null && mfaSecret !== void 0 ? mfaSecret : '', mfaRecoveryCodes !== null && mfaRecoveryCodes !== void 0 ? mfaRecoveryCodes : []);
    }
    async enableMfa(userId) {
        await this.userRepository.update(userId, { mfaEnabled: true });
    }
    encryptRecoveryCodes(mfaRecoveryCodes) {
        return mfaRecoveryCodes.map((code) => this.cipher.encrypt(code));
    }
    async disableMfa(userId) {
        await this.userRepository.update(userId, {
            mfaEnabled: false,
            mfaSecret: null,
            mfaRecoveryCodes: [],
        });
    }
};
exports.MfaService = MfaService;
exports.MfaService = MfaService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [user_repository_1.UserRepository,
        totp_service_1.TOTPService,
        n8n_core_1.Cipher])
], MfaService);
//# sourceMappingURL=mfa.service.js.map