"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CredentialsController = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const config_1 = __importDefault(require("../config"));
const credentials_service_1 = require("./credentials.service");
const InternalHooks_1 = require("../InternalHooks");
const Logger_1 = require("../Logger");
const not_found_error_1 = require("../errors/response-errors/not-found.error");
const unauthorized_error_1 = require("../errors/response-errors/unauthorized.error");
const naming_service_1 = require("../services/naming.service");
const License_1 = require("../License");
const credentials_repository_1 = require("../databases/repositories/credentials.repository");
const ownership_service_1 = require("../services/ownership.service");
const credentials_service_ee_1 = require("./credentials.service.ee");
const decorators_1 = require("../decorators");
const bad_request_error_1 = require("../errors/response-errors/bad-request.error");
const email_1 = require("../UserManagement/email");
const Db = __importStar(require("../Db"));
const utils = __importStar(require("../utils"));
const middlewares_1 = require("../middlewares");
let CredentialsController = class CredentialsController {
    constructor(credentialsService, enterpriseCredentialsService, credentialsRepository, namingService, license, logger, ownershipService, internalHooks, userManagementMailer) {
        this.credentialsService = credentialsService;
        this.enterpriseCredentialsService = enterpriseCredentialsService;
        this.credentialsRepository = credentialsRepository;
        this.namingService = namingService;
        this.license = license;
        this.logger = logger;
        this.ownershipService = ownershipService;
        this.internalHooks = internalHooks;
        this.userManagementMailer = userManagementMailer;
    }
    async getMany(req) {
        return await this.credentialsService.getMany(req.user, {
            listQueryOptions: req.listQueryOptions,
        });
    }
    async generateUniqueName(req) {
        var _a;
        const requestedName = (_a = req.query.name) !== null && _a !== void 0 ? _a : config_1.default.getEnv('credentials.defaultName');
        return {
            name: await this.namingService.getUniqueCredentialName(requestedName),
        };
    }
    async getOne(req) {
        var _a;
        if (this.license.isSharingEnabled()) {
            const { id: credentialId } = req.params;
            const includeDecryptedData = req.query.includeData === 'true';
            let credential = await this.credentialsRepository.findOne({
                where: { id: credentialId },
                relations: ['shared', 'shared.user'],
            });
            if (!credential) {
                throw new not_found_error_1.NotFoundError('Could not load the credential. If you think this is an error, ask the owner to share it with you again');
            }
            const userSharing = (_a = credential.shared) === null || _a === void 0 ? void 0 : _a.find((shared) => shared.user.id === req.user.id);
            if (!userSharing && !req.user.hasGlobalScope('credential:read')) {
                throw new unauthorized_error_1.UnauthorizedError('Forbidden.');
            }
            credential = this.ownershipService.addOwnedByAndSharedWith(credential);
            if (!includeDecryptedData || (userSharing === null || userSharing === void 0 ? void 0 : userSharing.role) === 'credential:user') {
                const { data: _, ...rest } = credential;
                return { ...rest };
            }
            const { data: _, ...rest } = credential;
            const decryptedData = this.credentialsService.redact(this.credentialsService.decrypt(credential), credential);
            return { data: decryptedData, ...rest };
        }
        const { id: credentialId } = req.params;
        const includeDecryptedData = req.query.includeData === 'true';
        const sharing = await this.credentialsService.getSharing(req.user, credentialId, { allowGlobalScope: true, globalScope: 'credential:read' }, ['credentials']);
        if (!sharing) {
            throw new not_found_error_1.NotFoundError(`Credential with ID "${credentialId}" could not be found.`);
        }
        const { credentials: credential } = sharing;
        const { data: _, ...rest } = credential;
        if (!includeDecryptedData) {
            return { ...rest };
        }
        const decryptedData = this.credentialsService.redact(this.credentialsService.decrypt(credential), credential);
        return { data: decryptedData, ...rest };
    }
    async testCredentials(req) {
        if (this.license.isSharingEnabled()) {
            const { credentials } = req.body;
            const credentialId = credentials.id;
            const { ownsCredential } = await this.enterpriseCredentialsService.isOwned(req.user, credentialId);
            const sharing = await this.enterpriseCredentialsService.getSharing(req.user, credentialId, {
                allowGlobalScope: true,
                globalScope: 'credential:read',
            });
            if (!ownsCredential) {
                if (!sharing) {
                    throw new unauthorized_error_1.UnauthorizedError('Forbidden');
                }
                const decryptedData = this.credentialsService.decrypt(sharing.credentials);
                Object.assign(credentials, { data: decryptedData });
            }
            const mergedCredentials = (0, n8n_workflow_1.deepCopy)(credentials);
            if (mergedCredentials.data && (sharing === null || sharing === void 0 ? void 0 : sharing.credentials)) {
                const decryptedData = this.credentialsService.decrypt(sharing.credentials);
                mergedCredentials.data = this.credentialsService.unredact(mergedCredentials.data, decryptedData);
            }
            return await this.credentialsService.test(req.user, mergedCredentials);
        }
        const { credentials } = req.body;
        const sharing = await this.credentialsService.getSharing(req.user, credentials.id, {
            allowGlobalScope: true,
            globalScope: 'credential:read',
        });
        const mergedCredentials = (0, n8n_workflow_1.deepCopy)(credentials);
        if (mergedCredentials.data && (sharing === null || sharing === void 0 ? void 0 : sharing.credentials)) {
            const decryptedData = this.credentialsService.decrypt(sharing.credentials);
            mergedCredentials.data = this.credentialsService.unredact(mergedCredentials.data, decryptedData);
        }
        return await this.credentialsService.test(req.user, mergedCredentials);
    }
    async createCredentials(req) {
        const newCredential = await this.credentialsService.prepareCreateData(req.body);
        const encryptedData = this.credentialsService.createEncryptedData(null, newCredential);
        const credential = await this.credentialsService.save(newCredential, encryptedData, req.user);
        void this.internalHooks.onUserCreatedCredentials({
            user: req.user,
            credential_name: newCredential.name,
            credential_type: credential.type,
            credential_id: credential.id,
            public_api: false,
        });
        return credential;
    }
    async updateCredentials(req) {
        const { id: credentialId } = req.params;
        const sharing = await this.credentialsService.getSharing(req.user, credentialId, {
            allowGlobalScope: true,
            globalScope: 'credential:update',
        }, ['credentials']);
        if (!sharing) {
            this.logger.info('Attempt to update credential blocked due to lack of permissions', {
                credentialId,
                userId: req.user.id,
            });
            throw new not_found_error_1.NotFoundError('Credential to be updated not found. You can only update credentials owned by you');
        }
        if (sharing.role !== 'credential:owner' && !req.user.hasGlobalScope('credential:update')) {
            this.logger.info('Attempt to update credential blocked due to lack of permissions', {
                credentialId,
                userId: req.user.id,
            });
            throw new unauthorized_error_1.UnauthorizedError('You can only update credentials owned by you');
        }
        const { credentials: credential } = sharing;
        const decryptedData = this.credentialsService.decrypt(credential);
        const preparedCredentialData = await this.credentialsService.prepareUpdateData(req.body, decryptedData);
        const newCredentialData = this.credentialsService.createEncryptedData(credentialId, preparedCredentialData);
        const responseData = await this.credentialsService.update(credentialId, newCredentialData);
        if (responseData === null) {
            throw new not_found_error_1.NotFoundError(`Credential ID "${credentialId}" could not be found to be updated.`);
        }
        const { data: _, ...rest } = responseData;
        this.logger.verbose('Credential updated', { credentialId });
        void this.internalHooks.onUserUpdatedCredentials({
            user: req.user,
            credential_name: credential.name,
            credential_type: credential.type,
            credential_id: credential.id,
        });
        return { ...rest };
    }
    async deleteCredentials(req) {
        const { id: credentialId } = req.params;
        const sharing = await this.credentialsService.getSharing(req.user, credentialId, {
            allowGlobalScope: true,
            globalScope: 'credential:delete',
        }, ['credentials']);
        if (!sharing) {
            this.logger.info('Attempt to delete credential blocked due to lack of permissions', {
                credentialId,
                userId: req.user.id,
            });
            throw new not_found_error_1.NotFoundError('Credential to be deleted not found. You can only removed credentials owned by you');
        }
        if (sharing.role !== 'credential:owner' && !req.user.hasGlobalScope('credential:delete')) {
            this.logger.info('Attempt to delete credential blocked due to lack of permissions', {
                credentialId,
                userId: req.user.id,
            });
            throw new unauthorized_error_1.UnauthorizedError('You can only remove credentials owned by you');
        }
        const { credentials: credential } = sharing;
        await this.credentialsService.delete(credential);
        void this.internalHooks.onUserDeletedCredentials({
            user: req.user,
            credential_name: credential.name,
            credential_type: credential.type,
            credential_id: credential.id,
        });
        return true;
    }
    async shareCredentials(req) {
        const { id: credentialId } = req.params;
        const { shareWithIds } = req.body;
        if (!Array.isArray(shareWithIds) ||
            !shareWithIds.every((userId) => typeof userId === 'string')) {
            throw new bad_request_error_1.BadRequestError('Bad request');
        }
        const isOwnedRes = await this.enterpriseCredentialsService.isOwned(req.user, credentialId);
        const { ownsCredential } = isOwnedRes;
        let { credential } = isOwnedRes;
        if (!ownsCredential || !credential) {
            credential = undefined;
            if (req.user.hasGlobalScope('credential:share')) {
                const sharedRes = await this.enterpriseCredentialsService.getSharing(req.user, credentialId, {
                    allowGlobalScope: true,
                    globalScope: 'credential:share',
                });
                credential = sharedRes === null || sharedRes === void 0 ? void 0 : sharedRes.credentials;
            }
            if (!credential) {
                throw new unauthorized_error_1.UnauthorizedError('Forbidden');
            }
        }
        const ownerIds = (await this.enterpriseCredentialsService.getSharings(Db.getConnection().createEntityManager(), credentialId, ['shared']))
            .filter((e) => e.role === 'credential:owner')
            .map((e) => e.userId);
        let amountRemoved = null;
        let newShareeIds = [];
        await Db.transaction(async (trx) => {
            const { affected } = await this.credentialsRepository.pruneSharings(trx, credentialId, [
                ...ownerIds,
                ...shareWithIds,
            ]);
            if (affected)
                amountRemoved = affected;
            const sharings = await this.enterpriseCredentialsService.getSharings(trx, credentialId);
            newShareeIds = utils.rightDiff([sharings, (sharing) => sharing.userId], [shareWithIds, (shareeId) => shareeId]);
            if (newShareeIds.length) {
                await this.enterpriseCredentialsService.share(trx, credential, newShareeIds);
            }
        });
        void this.internalHooks.onUserSharedCredentials({
            user: req.user,
            credential_name: credential.name,
            credential_type: credential.type,
            credential_id: credential.id,
            user_id_sharer: req.user.id,
            user_ids_sharees_added: newShareeIds,
            sharees_removed: amountRemoved,
        });
        await this.userManagementMailer.notifyCredentialsShared({
            sharer: req.user,
            newShareeIds,
            credentialsName: credential.name,
        });
    }
};
exports.CredentialsController = CredentialsController;
__decorate([
    (0, decorators_1.Get)('/', { middlewares: middlewares_1.listQueryMiddleware }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "getMany", null);
__decorate([
    (0, decorators_1.Get)('/new'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "generateUniqueName", null);
__decorate([
    (0, decorators_1.Get)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "getOne", null);
__decorate([
    (0, decorators_1.Post)('/test'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "testCredentials", null);
__decorate([
    (0, decorators_1.Post)('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "createCredentials", null);
__decorate([
    (0, decorators_1.Patch)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "updateCredentials", null);
__decorate([
    (0, decorators_1.Delete)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "deleteCredentials", null);
__decorate([
    (0, decorators_1.Licensed)('feat:sharing'),
    (0, decorators_1.Put)('/:id/share'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CredentialsController.prototype, "shareCredentials", null);
exports.CredentialsController = CredentialsController = __decorate([
    (0, decorators_1.RestController)('/credentials'),
    __metadata("design:paramtypes", [credentials_service_1.CredentialsService,
        credentials_service_ee_1.EnterpriseCredentialsService,
        credentials_repository_1.CredentialsRepository,
        naming_service_1.NamingService,
        License_1.License,
        Logger_1.Logger,
        ownership_service_1.OwnershipService,
        InternalHooks_1.InternalHooks,
        email_1.UserManagementMailer])
], CredentialsController);
//# sourceMappingURL=credentials.controller.js.map