"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExecutionRepository = void 0;
const typedi_1 = require("typedi");
const typeorm_1 = require("@n8n/typeorm");
const DateUtils_1 = require("@n8n/typeorm/util/DateUtils");
const flatted_1 = require("flatted");
const n8n_workflow_1 = require("n8n-workflow");
const n8n_core_1 = require("n8n-core");
const config_1 = __importDefault(require("../../config"));
const executionHelpers_1 = require("../../executions/executionHelpers");
const ExecutionEntity_1 = require("../entities/ExecutionEntity");
const ExecutionMetadata_1 = require("../entities/ExecutionMetadata");
const executionData_repository_1 = require("./executionData.repository");
const Logger_1 = require("../../Logger");
function parseFiltersToQueryBuilder(qb, filters) {
    if (filters === null || filters === void 0 ? void 0 : filters.status) {
        qb.andWhere('execution.status IN (:...workflowStatus)', {
            workflowStatus: filters.status,
        });
    }
    if (filters === null || filters === void 0 ? void 0 : filters.finished) {
        qb.andWhere({ finished: filters.finished });
    }
    if ((filters === null || filters === void 0 ? void 0 : filters.metadata) && (0, executionHelpers_1.isAdvancedExecutionFiltersEnabled)()) {
        qb.leftJoin(ExecutionMetadata_1.ExecutionMetadata, 'md', 'md.executionId = execution.id');
        for (const md of filters.metadata) {
            qb.andWhere('md.key = :key AND md.value = :value', md);
        }
    }
    if (filters === null || filters === void 0 ? void 0 : filters.startedAfter) {
        qb.andWhere({
            startedAt: (0, typeorm_1.MoreThanOrEqual)(DateUtils_1.DateUtils.mixedDateToUtcDatetimeString(new Date(filters.startedAfter))),
        });
    }
    if (filters === null || filters === void 0 ? void 0 : filters.startedBefore) {
        qb.andWhere({
            startedAt: (0, typeorm_1.LessThanOrEqual)(DateUtils_1.DateUtils.mixedDateToUtcDatetimeString(new Date(filters.startedBefore))),
        });
    }
    if (filters === null || filters === void 0 ? void 0 : filters.workflowId) {
        qb.andWhere({
            workflowId: filters.workflowId,
        });
    }
}
let ExecutionRepository = class ExecutionRepository extends typeorm_1.Repository {
    constructor(dataSource, logger, executionDataRepository, binaryDataService) {
        super(ExecutionEntity_1.ExecutionEntity, dataSource.manager);
        this.logger = logger;
        this.executionDataRepository = executionDataRepository;
        this.binaryDataService = binaryDataService;
        this.hardDeletionBatchSize = 100;
    }
    async findMultipleExecutions(queryParams, options) {
        if (options === null || options === void 0 ? void 0 : options.includeData) {
            if (!queryParams.relations) {
                queryParams.relations = [];
            }
            queryParams.relations.push('executionData');
        }
        const executions = await this.find(queryParams);
        if ((options === null || options === void 0 ? void 0 : options.includeData) && (options === null || options === void 0 ? void 0 : options.unflattenData)) {
            return executions.map((execution) => {
                const { executionData, ...rest } = execution;
                return {
                    ...rest,
                    data: (0, flatted_1.parse)(executionData.data),
                    workflowData: executionData.workflowData,
                };
            });
        }
        else if (options === null || options === void 0 ? void 0 : options.includeData) {
            return executions.map((execution) => {
                const { executionData, ...rest } = execution;
                return {
                    ...rest,
                    data: execution.executionData.data,
                    workflowData: execution.executionData.workflowData,
                };
            });
        }
        return executions.map((execution) => {
            const { executionData, ...rest } = execution;
            return rest;
        });
    }
    async findSingleExecution(id, options) {
        const findOptions = {
            where: {
                id,
                ...options === null || options === void 0 ? void 0 : options.where,
            },
        };
        if (options === null || options === void 0 ? void 0 : options.includeData) {
            findOptions.relations = ['executionData'];
        }
        const execution = await this.findOne(findOptions);
        if (!execution) {
            return undefined;
        }
        const { executionData, ...rest } = execution;
        if ((options === null || options === void 0 ? void 0 : options.includeData) && (options === null || options === void 0 ? void 0 : options.unflattenData)) {
            return {
                ...rest,
                data: (0, flatted_1.parse)(execution.executionData.data),
                workflowData: execution.executionData.workflowData,
            };
        }
        else if (options === null || options === void 0 ? void 0 : options.includeData) {
            return {
                ...rest,
                data: execution.executionData.data,
                workflowData: execution.executionData.workflowData,
            };
        }
        return rest;
    }
    async createNewExecution(execution) {
        const { data, workflowData, ...rest } = execution;
        const { identifiers: inserted } = await this.insert(rest);
        const { id: executionId } = inserted[0];
        const { connections, nodes, name, settings } = workflowData !== null && workflowData !== void 0 ? workflowData : {};
        await this.executionDataRepository.insert({
            executionId,
            workflowData: { connections, nodes, name, settings, id: workflowData.id },
            data: (0, flatted_1.stringify)(data),
        });
        return String(executionId);
    }
    async markAsCrashed(executionIds) {
        await this.update({ id: (0, typeorm_1.In)(executionIds) }, {
            status: 'crashed',
            stoppedAt: new Date(),
        });
    }
    async hardDelete(ids) {
        return await Promise.all([
            this.delete(ids.executionId),
            this.binaryDataService.deleteMany([ids]),
        ]);
    }
    async updateStatus(executionId, status) {
        await this.update({ id: executionId }, { status });
    }
    async updateExistingExecution(executionId, execution) {
        const { id, data, workflowId, workflowData, startedAt, ...executionInformation } = execution;
        if (Object.keys(executionInformation).length > 0) {
            await this.update({ id: executionId }, executionInformation);
        }
        if (data || workflowData) {
            const executionData = {};
            if (workflowData) {
                executionData.workflowData = workflowData;
            }
            if (data) {
                executionData.data = (0, flatted_1.stringify)(data);
            }
            await this.executionDataRepository.update({ executionId }, executionData);
        }
    }
    async countExecutions(filters, accessibleWorkflowIds, currentlyRunningExecutions, hasGlobalRead) {
        const dbType = config_1.default.getEnv('database.type');
        if (dbType !== 'postgresdb' || (filters && Object.keys(filters).length > 0) || !hasGlobalRead) {
            const query = this.createQueryBuilder('execution').andWhere('execution.workflowId IN (:...accessibleWorkflowIds)', { accessibleWorkflowIds });
            if (currentlyRunningExecutions.length > 0) {
                query.andWhere('execution.id NOT IN (:...currentlyRunningExecutions)', {
                    currentlyRunningExecutions,
                });
            }
            parseFiltersToQueryBuilder(query, filters);
            const count = await query.getCount();
            return { count, estimated: false };
        }
        try {
            const estimateRowsNumberSql = "SELECT n_live_tup FROM pg_stat_all_tables WHERE relname = 'execution_entity';";
            const rows = (await this.query(estimateRowsNumberSql));
            const estimate = parseInt(rows[0].n_live_tup, 10);
            if (estimate > 100000) {
                return { count: estimate, estimated: true };
            }
        }
        catch (error) {
            if (error instanceof Error) {
                this.logger.warn(`Failed to get executions count from Postgres: ${error.message}`, {
                    error,
                });
            }
        }
        const count = await this.count({
            where: {
                workflowId: (0, typeorm_1.In)(accessibleWorkflowIds),
            },
        });
        return { count, estimated: false };
    }
    async searchExecutions(filters, limit, excludedExecutionIds, accessibleWorkflowIds, additionalFilters) {
        if (accessibleWorkflowIds.length === 0) {
            return [];
        }
        const query = this.createQueryBuilder('execution')
            .select([
            'execution.id',
            'execution.finished',
            'execution.mode',
            'execution.retryOf',
            'execution.retrySuccessId',
            'execution.status',
            'execution.startedAt',
            'execution.stoppedAt',
            'execution.workflowId',
            'execution.waitTill',
            'workflow.name',
        ])
            .innerJoin('execution.workflow', 'workflow')
            .limit(limit)
            .orderBy({ 'execution.id': 'DESC' })
            .andWhere('execution.workflowId IN (:...accessibleWorkflowIds)', { accessibleWorkflowIds });
        if (excludedExecutionIds.length > 0) {
            query.andWhere('execution.id NOT IN (:...excludedExecutionIds)', { excludedExecutionIds });
        }
        if (additionalFilters === null || additionalFilters === void 0 ? void 0 : additionalFilters.lastId) {
            query.andWhere('execution.id < :lastId', { lastId: additionalFilters.lastId });
        }
        if (additionalFilters === null || additionalFilters === void 0 ? void 0 : additionalFilters.firstId) {
            query.andWhere('execution.id > :firstId', { firstId: additionalFilters.firstId });
        }
        parseFiltersToQueryBuilder(query, filters);
        const executions = await query.getMany();
        return executions.map((execution) => {
            const { workflow, waitTill, ...rest } = execution;
            return {
                ...rest,
                waitTill: waitTill !== null && waitTill !== void 0 ? waitTill : undefined,
                workflowName: workflow.name,
            };
        });
    }
    async deleteExecutionsByFilter(filters, accessibleWorkflowIds, deleteConditions) {
        if (!(deleteConditions === null || deleteConditions === void 0 ? void 0 : deleteConditions.deleteBefore) && !(deleteConditions === null || deleteConditions === void 0 ? void 0 : deleteConditions.ids)) {
            throw new n8n_workflow_1.ApplicationError('Either "deleteBefore" or "ids" must be present in the request body');
        }
        const query = this.createQueryBuilder('execution')
            .select(['execution.id', 'execution.workflowId'])
            .andWhere('execution.workflowId IN (:...accessibleWorkflowIds)', { accessibleWorkflowIds });
        if (deleteConditions.deleteBefore) {
            query.andWhere('execution.startedAt <= :deleteBefore', {
                deleteBefore: deleteConditions.deleteBefore,
            });
            parseFiltersToQueryBuilder(query, filters);
        }
        else if (deleteConditions.ids) {
            query.andWhere('execution.id IN (:...executionIds)', { executionIds: deleteConditions.ids });
        }
        const executions = await query.getMany();
        if (!executions.length) {
            if (deleteConditions.ids) {
                this.logger.error('Failed to delete an execution due to insufficient permissions', {
                    executionIds: deleteConditions.ids,
                });
            }
            return;
        }
        const ids = executions.map(({ id, workflowId }) => ({
            executionId: id,
            workflowId,
        }));
        do {
            const batch = ids.splice(0, this.hardDeletionBatchSize);
            await Promise.all([
                this.delete(batch.map(({ executionId }) => executionId)),
                this.binaryDataService.deleteMany(batch),
            ]);
        } while (ids.length > 0);
    }
    async getIdsSince(date) {
        return await this.find({
            select: ['id'],
            where: {
                startedAt: (0, typeorm_1.MoreThanOrEqual)(DateUtils_1.DateUtils.mixedDateToUtcDatetimeString(date)),
            },
        }).then((executions) => executions.map(({ id }) => id));
    }
    async softDeletePrunableExecutions() {
        const maxAge = config_1.default.getEnv('executions.pruneDataMaxAge');
        const maxCount = config_1.default.getEnv('executions.pruneDataMaxCount');
        const date = new Date();
        date.setHours(date.getHours() - maxAge);
        const toPrune = [
            { stoppedAt: (0, typeorm_1.LessThanOrEqual)(DateUtils_1.DateUtils.mixedDateToUtcDatetimeString(date)) },
        ];
        if (maxCount > 0) {
            const executions = await this.find({
                select: ['id'],
                skip: maxCount,
                take: 1,
                order: { id: 'DESC' },
            });
            if (executions[0]) {
                toPrune.push({ id: (0, typeorm_1.LessThanOrEqual)(executions[0].id) });
            }
        }
        const [timeBasedWhere, countBasedWhere] = toPrune;
        return await this.createQueryBuilder()
            .update(ExecutionEntity_1.ExecutionEntity)
            .set({ deletedAt: new Date() })
            .where({
            deletedAt: (0, typeorm_1.IsNull)(),
            status: (0, typeorm_1.Not)((0, typeorm_1.In)(['new', 'running', 'waiting'])),
        })
            .andWhere(new typeorm_1.Brackets((qb) => countBasedWhere
            ? qb.where(timeBasedWhere).orWhere(countBasedWhere)
            : qb.where(timeBasedWhere)))
            .execute();
    }
    async hardDeleteSoftDeletedExecutions() {
        const date = new Date();
        date.setHours(date.getHours() - config_1.default.getEnv('executions.pruneDataHardDeleteBuffer'));
        const workflowIdsAndExecutionIds = (await this.find({
            select: ['workflowId', 'id'],
            where: {
                deletedAt: (0, typeorm_1.LessThanOrEqual)(DateUtils_1.DateUtils.mixedDateToUtcDatetimeString(date)),
            },
            take: this.hardDeletionBatchSize,
            withDeleted: true,
        })).map(({ id: executionId, workflowId }) => ({ workflowId, executionId }));
        return workflowIdsAndExecutionIds;
    }
    async deleteByIds(executionIds) {
        return await this.delete({ id: (0, typeorm_1.In)(executionIds) });
    }
    async getWaitingExecutions() {
        const waitTill = new Date(Date.now() + 70000);
        const where = {
            waitTill: (0, typeorm_1.LessThanOrEqual)(waitTill),
            status: (0, typeorm_1.Not)('crashed'),
        };
        const dbType = config_1.default.getEnv('database.type');
        if (dbType === 'sqlite') {
            where.waitTill = (0, typeorm_1.LessThanOrEqual)(DateUtils_1.DateUtils.mixedDateToUtcDatetimeString(waitTill));
        }
        return await this.findMultipleExecutions({
            select: ['id', 'waitTill'],
            where,
            order: {
                waitTill: 'ASC',
            },
        });
    }
    async getExecutionsCountForPublicApi(data) {
        const executions = await this.count({
            where: {
                ...(data.lastId && { id: (0, typeorm_1.LessThan)(data.lastId) }),
                ...(data.status && { ...this.getStatusCondition(data.status) }),
                ...(data.workflowIds && { workflowId: (0, typeorm_1.In)(data.workflowIds) }),
                ...(data.excludedWorkflowIds && { workflowId: (0, typeorm_1.Not)((0, typeorm_1.In)(data.excludedWorkflowIds)) }),
            },
            take: data.limit,
        });
        return executions;
    }
    getStatusCondition(status) {
        const condition = {};
        if (status === 'success') {
            condition.status = 'success';
        }
        else if (status === 'waiting') {
            condition.status = 'waiting';
        }
        else if (status === 'error') {
            condition.status = (0, typeorm_1.In)(['error', 'crashed']);
        }
        return condition;
    }
    async getExecutionsForPublicApi(params) {
        var _a, _b;
        let where = {};
        if (params.lastId && ((_a = params.excludedExecutionsIds) === null || _a === void 0 ? void 0 : _a.length)) {
            where.id = (0, typeorm_1.Raw)((id) => `${id} < :lastId AND ${id} NOT IN (:...excludedExecutionsIds)`, {
                lastId: params.lastId,
                excludedExecutionsIds: params.excludedExecutionsIds,
            });
        }
        else if (params.lastId) {
            where.id = (0, typeorm_1.LessThan)(params.lastId);
        }
        else if ((_b = params.excludedExecutionsIds) === null || _b === void 0 ? void 0 : _b.length) {
            where.id = (0, typeorm_1.Not)((0, typeorm_1.In)(params.excludedExecutionsIds));
        }
        if (params.status) {
            where = { ...where, ...this.getStatusCondition(params.status) };
        }
        if (params.workflowIds) {
            where = { ...where, workflowId: (0, typeorm_1.In)(params.workflowIds) };
        }
        return await this.findMultipleExecutions({
            select: [
                'id',
                'mode',
                'retryOf',
                'retrySuccessId',
                'startedAt',
                'stoppedAt',
                'workflowId',
                'waitTill',
                'finished',
            ],
            where,
            order: { id: 'DESC' },
            take: params.limit,
            relations: ['executionData'],
        }, {
            includeData: params.includeData,
            unflattenData: true,
        });
    }
    async getExecutionInWorkflowsForPublicApi(id, workflowIds, includeData) {
        return await this.findSingleExecution(id, {
            where: {
                workflowId: (0, typeorm_1.In)(workflowIds),
            },
            includeData,
            unflattenData: true,
        });
    }
    async findWithUnflattenedData(executionId, accessibleWorkflowIds) {
        return await this.findSingleExecution(executionId, {
            where: {
                workflowId: (0, typeorm_1.In)(accessibleWorkflowIds),
            },
            includeData: true,
            unflattenData: true,
        });
    }
    async findIfShared(executionId, sharedWorkflowIds) {
        return await this.findSingleExecution(executionId, {
            where: {
                workflowId: (0, typeorm_1.In)(sharedWorkflowIds),
            },
            includeData: true,
            unflattenData: false,
        });
    }
    async findIfAccessible(executionId, accessibleWorkflowIds) {
        return await this.findSingleExecution(executionId, {
            where: { workflowId: (0, typeorm_1.In)(accessibleWorkflowIds) },
        });
    }
    async getManyActive(activeExecutionIds, accessibleWorkflowIds, filter) {
        const where = {
            id: (0, typeorm_1.In)(activeExecutionIds),
            status: (0, typeorm_1.Not)((0, typeorm_1.In)(['finished', 'stopped', 'error', 'crashed'])),
        };
        if (filter) {
            const { workflowId, status, finished } = filter;
            if (workflowId && accessibleWorkflowIds.includes(workflowId)) {
                where.workflowId = workflowId;
            }
            else {
                where.workflowId = (0, typeorm_1.In)(accessibleWorkflowIds);
            }
            if (status) {
                where.status = (0, typeorm_1.In)(status);
            }
            if (finished !== undefined) {
                where.finished = finished;
            }
        }
        else {
            where.workflowId = (0, typeorm_1.In)(accessibleWorkflowIds);
        }
        return await this.findMultipleExecutions({
            select: ['id', 'workflowId', 'mode', 'retryOf', 'startedAt', 'stoppedAt', 'status'],
            order: { id: 'DESC' },
            where,
        });
    }
};
exports.ExecutionRepository = ExecutionRepository;
exports.ExecutionRepository = ExecutionRepository = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [typeorm_1.DataSource,
        Logger_1.Logger,
        executionData_repository_1.ExecutionDataRepository,
        n8n_core_1.BinaryDataService])
], ExecutionRepository);
//# sourceMappingURL=execution.repository.js.map