"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SharedCredentialsRepository = void 0;
const typedi_1 = require("typedi");
const typeorm_1 = require("@n8n/typeorm");
const SharedCredentials_1 = require("../entities/SharedCredentials");
let SharedCredentialsRepository = class SharedCredentialsRepository extends typeorm_1.Repository {
    constructor(dataSource) {
        super(SharedCredentials_1.SharedCredentials, dataSource.manager);
    }
    async findCredentialForUser(credentialsId, user) {
        const sharedCredential = await this.findOne({
            relations: ['credentials'],
            where: {
                credentialsId,
                ...(!user.hasGlobalScope('credential:read') ? { userId: user.id } : {}),
            },
        });
        if (!sharedCredential)
            return null;
        return sharedCredential.credentials;
    }
    async findByCredentialIds(credentialIds, role) {
        return await this.find({
            relations: ['credentials', 'user'],
            where: {
                credentialsId: (0, typeorm_1.In)(credentialIds),
                role,
            },
        });
    }
    async makeOwnerOfAllCredentials(user) {
        return await this.update({ userId: (0, typeorm_1.Not)(user.id), role: 'credential:owner' }, { user });
    }
    async getOwnedCredentialIds(userIds) {
        return await this.getCredentialIdsByUserAndRole(userIds, ['credential:owner']);
    }
    async getAccessibleCredentialIds(userIds) {
        return await this.getCredentialIdsByUserAndRole(userIds, [
            'credential:owner',
            'credential:user',
        ]);
    }
    async getCredentialIdsByUserAndRole(userIds, roles) {
        const sharings = await this.find({
            where: {
                userId: (0, typeorm_1.In)(userIds),
                role: (0, typeorm_1.In)(roles),
            },
        });
        return sharings.map((s) => s.credentialsId);
    }
    async deleteByIds(transaction, sharedCredentialsIds, user) {
        return await transaction.delete(SharedCredentials_1.SharedCredentials, {
            user,
            credentialsId: (0, typeorm_1.In)(sharedCredentialsIds),
        });
    }
};
exports.SharedCredentialsRepository = SharedCredentialsRepository;
exports.SharedCredentialsRepository = SharedCredentialsRepository = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [typeorm_1.DataSource])
], SharedCredentialsRepository);
//# sourceMappingURL=sharedCredentials.repository.js.map