"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SharedWorkflowRepository = void 0;
const typedi_1 = require("typedi");
const typeorm_1 = require("@n8n/typeorm");
const SharedWorkflow_1 = require("../entities/SharedWorkflow");
let SharedWorkflowRepository = class SharedWorkflowRepository extends typeorm_1.Repository {
    constructor(dataSource) {
        super(SharedWorkflow_1.SharedWorkflow, dataSource.manager);
    }
    async hasAccess(workflowId, user) {
        const where = {
            workflowId,
        };
        if (!user.hasGlobalScope('workflow:read')) {
            where.userId = user.id;
        }
        return await this.exist({ where });
    }
    async getSharedUserIds(workflowId) {
        const sharedWorkflows = await this.find({
            select: ['userId'],
            where: { workflowId },
        });
        return sharedWorkflows.map((sharing) => sharing.userId);
    }
    async getSharedWorkflowIds(workflowIds) {
        const sharedWorkflows = await this.find({
            select: ['workflowId'],
            where: {
                workflowId: (0, typeorm_1.In)(workflowIds),
            },
        });
        return sharedWorkflows.map((sharing) => sharing.workflowId);
    }
    async findByWorkflowIds(workflowIds) {
        return await this.find({
            relations: ['user'],
            where: {
                role: 'workflow:owner',
                workflowId: (0, typeorm_1.In)(workflowIds),
            },
        });
    }
    async findSharingRole(userId, workflowId) {
        return await this.findOne({
            select: ['role'],
            where: { workflowId, userId },
        }).then((shared) => shared === null || shared === void 0 ? void 0 : shared.role);
    }
    async findSharing(workflowId, user, scope, { roles, extraRelations } = {}) {
        const where = {
            workflow: { id: workflowId },
        };
        if (!user.hasGlobalScope(scope)) {
            where.user = { id: user.id };
        }
        if (roles) {
            where.role = (0, typeorm_1.In)(roles);
        }
        const relations = ['workflow'];
        if (extraRelations)
            relations.push(...extraRelations);
        return await this.findOne({ relations, where });
    }
    async makeOwnerOfAllWorkflows(user) {
        return await this.update({ userId: (0, typeorm_1.Not)(user.id), role: 'workflow:owner' }, { user });
    }
    async getSharing(user, workflowId, options, relations = ['workflow']) {
        const where = { workflowId };
        if (!options.allowGlobalScope || !user.hasGlobalScope(options.globalScope)) {
            where.userId = user.id;
        }
        return await this.findOne({ where, relations });
    }
    async getSharedWorkflows(user, options) {
        return await this.find({
            where: {
                ...(!['global:owner', 'global:admin'].includes(user.role) && { userId: user.id }),
                ...(options.workflowIds && { workflowId: (0, typeorm_1.In)(options.workflowIds) }),
            },
            ...(options.relations && { relations: options.relations }),
        });
    }
    async share(transaction, workflow, users) {
        const newSharedWorkflows = users.reduce((acc, user) => {
            if (user.isPending) {
                return acc;
            }
            const entity = {
                workflowId: workflow.id,
                userId: user.id,
                role: 'workflow:editor',
            };
            acc.push(this.create(entity));
            return acc;
        }, []);
        return await transaction.save(newSharedWorkflows);
    }
    async findWithFields(workflowIds, { select }) {
        return await this.find({
            where: {
                workflowId: (0, typeorm_1.In)(workflowIds),
            },
            select,
        });
    }
    async deleteByIds(transaction, sharedWorkflowIds, user) {
        return await transaction.delete(SharedWorkflow_1.SharedWorkflow, {
            user,
            workflowId: (0, typeorm_1.In)(sharedWorkflowIds),
        });
    }
};
exports.SharedWorkflowRepository = SharedWorkflowRepository;
exports.SharedWorkflowRepository = SharedWorkflowRepository = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [typeorm_1.DataSource])
], SharedWorkflowRepository);
//# sourceMappingURL=sharedWorkflow.repository.js.map