"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActiveExecutionService = void 0;
const typedi_1 = require("typedi");
const ActiveExecutions_1 = require("../ActiveExecutions");
const Logger_1 = require("../Logger");
const Queue_1 = require("../Queue");
const WaitTracker_1 = require("../WaitTracker");
const execution_repository_1 = require("../databases/repositories/execution.repository");
const executionHelpers_1 = require("../executions/executionHelpers");
const config_1 = __importDefault(require("../config"));
let ActiveExecutionService = class ActiveExecutionService {
    constructor(logger, queue, activeExecutions, executionRepository, waitTracker) {
        this.logger = logger;
        this.queue = queue;
        this.activeExecutions = activeExecutions;
        this.executionRepository = executionRepository;
        this.waitTracker = waitTracker;
        this.isRegularMode = config_1.default.getEnv('executions.mode') === 'regular';
    }
    async findOne(executionId, accessibleWorkflowIds) {
        return await this.executionRepository.findIfAccessible(executionId, accessibleWorkflowIds);
    }
    toSummary(execution) {
        var _a;
        return {
            id: execution.id,
            workflowId: (_a = execution.workflowId) !== null && _a !== void 0 ? _a : '',
            mode: execution.mode,
            retryOf: execution.retryOf !== null ? execution.retryOf : undefined,
            startedAt: new Date(execution.startedAt),
            status: execution.status,
            stoppedAt: 'stoppedAt' in execution ? execution.stoppedAt : undefined,
        };
    }
    async findManyInRegularMode(filter, accessibleWorkflowIds) {
        return this.activeExecutions
            .getActiveExecutions()
            .filter(({ workflowId }) => {
            if (filter.workflowId && filter.workflowId !== workflowId)
                return false;
            if (workflowId && !accessibleWorkflowIds.includes(workflowId))
                return false;
            return true;
        })
            .map((execution) => this.toSummary(execution))
            .sort((a, b) => Number(b.id) - Number(a.id));
    }
    async findManyInQueueMode(filter, accessibleWorkflowIds) {
        const activeManualExecutionIds = this.activeExecutions
            .getActiveExecutions()
            .map((execution) => execution.id);
        const activeJobs = await this.queue.getJobs(['active', 'waiting']);
        const activeProductionExecutionIds = activeJobs.map((job) => job.data.executionId);
        const activeExecutionIds = activeProductionExecutionIds.concat(activeManualExecutionIds);
        if (activeExecutionIds.length === 0)
            return [];
        const activeExecutions = await this.executionRepository.getManyActive(activeExecutionIds, accessibleWorkflowIds, filter);
        return activeExecutions.map((execution) => {
            if (!execution.status) {
                execution.status = (0, executionHelpers_1.getStatusUsingPreviousExecutionStatusMethod)(execution);
            }
            return this.toSummary(execution);
        });
    }
    async stop(execution) {
        const result = await this.activeExecutions.stopExecution(execution.id);
        if (result) {
            return {
                mode: result.mode,
                startedAt: new Date(result.startedAt),
                stoppedAt: result.stoppedAt ? new Date(result.stoppedAt) : undefined,
                finished: result.finished,
                status: result.status,
            };
        }
        if (this.isRegularMode)
            return await this.waitTracker.stopExecution(execution.id);
        try {
            return await this.waitTracker.stopExecution(execution.id);
        }
        catch { }
        const activeJobs = await this.queue.getJobs(['active', 'waiting']);
        const job = activeJobs.find(({ data }) => data.executionId === execution.id);
        if (!job) {
            this.logger.debug('Could not stop job because it is no longer in queue', {
                jobId: execution.id,
            });
        }
        else {
            await this.queue.stopJob(job);
        }
        return {
            mode: execution.mode,
            startedAt: new Date(execution.startedAt),
            stoppedAt: execution.stoppedAt ? new Date(execution.stoppedAt) : undefined,
            finished: execution.finished,
            status: execution.status,
        };
    }
};
exports.ActiveExecutionService = ActiveExecutionService;
exports.ActiveExecutionService = ActiveExecutionService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [Logger_1.Logger,
        Queue_1.Queue,
        ActiveExecutions_1.ActiveExecutions,
        execution_repository_1.ExecutionRepository,
        WaitTracker_1.WaitTracker])
], ActiveExecutionService);
//# sourceMappingURL=active-execution.service.js.map