"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OwnershipService = void 0;
const typedi_1 = require("typedi");
const cache_service_1 = require("../services/cache/cache.service");
const sharedWorkflow_repository_1 = require("../databases/repositories/sharedWorkflow.repository");
const user_repository_1 = require("../databases/repositories/user.repository");
let OwnershipService = class OwnershipService {
    constructor(cacheService, userRepository, sharedWorkflowRepository) {
        this.cacheService = cacheService;
        this.userRepository = userRepository;
        this.sharedWorkflowRepository = sharedWorkflowRepository;
    }
    async getWorkflowOwnerCached(workflowId) {
        const cachedValue = await this.cacheService.getHashValue('workflow-ownership', workflowId);
        if (cachedValue)
            return this.userRepository.create(cachedValue);
        const sharedWorkflow = await this.sharedWorkflowRepository.findOneOrFail({
            where: { workflowId, role: 'workflow:owner' },
            relations: ['user'],
        });
        void this.cacheService.setHash('workflow-ownership', { [workflowId]: sharedWorkflow.user });
        return sharedWorkflow.user;
    }
    addOwnedByAndSharedWith(rawEntity) {
        const { shared, ...rest } = rawEntity;
        const entity = rest;
        Object.assign(entity, { ownedBy: null, sharedWith: [] });
        shared === null || shared === void 0 ? void 0 : shared.forEach(({ user, role }) => {
            const { id, email, firstName, lastName } = user;
            if (role === 'credential:owner' || role === 'workflow:owner') {
                entity.ownedBy = { id, email, firstName, lastName };
            }
            else {
                entity.sharedWith.push({ id, email, firstName, lastName });
            }
        });
        return entity;
    }
    async getInstanceOwner() {
        return await this.userRepository.findOneOrFail({
            where: { role: 'global:owner' },
        });
    }
};
exports.OwnershipService = OwnershipService;
exports.OwnershipService = OwnershipService = __decorate([
    (0, typedi_1.Service)(),
    __metadata("design:paramtypes", [cache_service_1.CacheService,
        user_repository_1.UserRepository,
        sharedWorkflow_repository_1.SharedWorkflowRepository])
], OwnershipService);
//# sourceMappingURL=ownership.service.js.map