import { PostHogClient } from '../posthog';
import type { ITelemetryTrackProperties } from 'n8n-workflow';
import { InstanceSettings } from 'n8n-core';
import type { IExecutionTrackProperties } from '../Interfaces';
import { Logger } from '../Logger';
import { License } from '../License';
import { WorkflowRepository } from '../databases/repositories/workflow.repository';
interface IExecutionTrackData {
    count: number;
    first: Date;
}
interface IExecutionsBuffer {
    [workflowId: string]: {
        manual_error?: IExecutionTrackData;
        manual_success?: IExecutionTrackData;
        prod_error?: IExecutionTrackData;
        prod_success?: IExecutionTrackData;
        user_id: string | undefined;
    };
}
export declare class Telemetry {
    private readonly logger;
    private readonly postHog;
    private readonly license;
    private readonly instanceSettings;
    private readonly workflowRepository;
    private rudderStack?;
    private pulseIntervalReference;
    private executionCountsBuffer;
    constructor(logger: Logger, postHog: PostHogClient, license: License, instanceSettings: InstanceSettings, workflowRepository: WorkflowRepository);
    init(): Promise<void>;
    private startPulse;
    private pulse;
    trackWorkflowExecution(properties: IExecutionTrackProperties): Promise<void>;
    trackN8nStop(): Promise<void>;
    identify(traits?: {
        [key: string]: string | number | boolean | object | undefined | null;
    }): Promise<void>;
    track(eventName: string, properties?: ITelemetryTrackProperties, { withPostHog }?: {
        withPostHog: boolean;
    }): Promise<void>;
    getCountsBuffer(): IExecutionsBuffer;
}
export {};
