"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkflowsController = void 0;
const express_1 = __importDefault(require("express"));
const uuid_1 = require("uuid");
const axios_1 = __importDefault(require("axios"));
const Db = __importStar(require("../Db"));
const ResponseHelper = __importStar(require("../ResponseHelper"));
const WorkflowHelpers = __importStar(require("../WorkflowHelpers"));
const config_1 = __importDefault(require("../config"));
const decorators_1 = require("../decorators");
const SharedWorkflow_1 = require("../databases/entities/SharedWorkflow");
const WorkflowEntity_1 = require("../databases/entities/WorkflowEntity");
const sharedWorkflow_repository_1 = require("../databases/repositories/sharedWorkflow.repository");
const tag_repository_1 = require("../databases/repositories/tag.repository");
const workflow_repository_1 = require("../databases/repositories/workflow.repository");
const user_repository_1 = require("../databases/repositories/user.repository");
const GenericHelpers_1 = require("../GenericHelpers");
const ExternalHooks_1 = require("../ExternalHooks");
const workflow_service_1 = require("./workflow.service");
const License_1 = require("../License");
const InternalHooks_1 = require("../InternalHooks");
const utils = __importStar(require("../utils"));
const middlewares_1 = require("../middlewares");
const tag_service_1 = require("../services/tag.service");
const workflowHistory_service_ee_1 = require("./workflowHistory/workflowHistory.service.ee");
const Logger_1 = require("../Logger");
const bad_request_error_1 = require("../errors/response-errors/bad-request.error");
const not_found_error_1 = require("../errors/response-errors/not-found.error");
const internal_server_error_1 = require("../errors/response-errors/internal-server.error");
const unauthorized_error_1 = require("../errors/response-errors/unauthorized.error");
const naming_service_1 = require("../services/naming.service");
const userOnboarding_service_1 = require("../services/userOnboarding.service");
const credentials_service_1 = require("../credentials/credentials.service");
const workflow_service_ee_1 = require("./workflow.service.ee");
const workflowExecution_service_1 = require("./workflowExecution.service");
const workflowSharing_service_1 = require("./workflowSharing.service");
const email_1 = require("../UserManagement/email");
let WorkflowsController = class WorkflowsController {
    constructor(logger, internalHooks, externalHooks, tagRepository, enterpriseWorkflowService, workflowHistoryService, tagService, namingService, userOnboardingService, workflowRepository, workflowService, workflowExecutionService, workflowSharingService, sharedWorkflowRepository, userRepository, license, mailer, credentialsService) {
        this.logger = logger;
        this.internalHooks = internalHooks;
        this.externalHooks = externalHooks;
        this.tagRepository = tagRepository;
        this.enterpriseWorkflowService = enterpriseWorkflowService;
        this.workflowHistoryService = workflowHistoryService;
        this.tagService = tagService;
        this.namingService = namingService;
        this.userOnboardingService = userOnboardingService;
        this.workflowRepository = workflowRepository;
        this.workflowService = workflowService;
        this.workflowExecutionService = workflowExecutionService;
        this.workflowSharingService = workflowSharingService;
        this.sharedWorkflowRepository = sharedWorkflowRepository;
        this.userRepository = userRepository;
        this.license = license;
        this.mailer = mailer;
        this.credentialsService = credentialsService;
    }
    async create(req) {
        delete req.body.id;
        const newWorkflow = new WorkflowEntity_1.WorkflowEntity();
        Object.assign(newWorkflow, req.body);
        newWorkflow.versionId = (0, uuid_1.v4)();
        await (0, GenericHelpers_1.validateEntity)(newWorkflow);
        await this.externalHooks.run('workflow.create', [newWorkflow]);
        const { tags: tagIds } = req.body;
        if ((tagIds === null || tagIds === void 0 ? void 0 : tagIds.length) && !config_1.default.getEnv('workflowTagsDisabled')) {
            newWorkflow.tags = await this.tagRepository.findMany(tagIds);
        }
        await WorkflowHelpers.replaceInvalidCredentials(newWorkflow);
        WorkflowHelpers.addNodeIds(newWorkflow);
        if (this.license.isSharingEnabled()) {
            const allCredentials = await this.credentialsService.getMany(req.user);
            try {
                this.enterpriseWorkflowService.validateCredentialPermissionsToUser(newWorkflow, allCredentials);
            }
            catch (error) {
                throw new bad_request_error_1.BadRequestError('The workflow you are trying to save contains credentials that are not shared with you');
            }
        }
        let savedWorkflow;
        await Db.transaction(async (transactionManager) => {
            savedWorkflow = await transactionManager.save(newWorkflow);
            const newSharedWorkflow = new SharedWorkflow_1.SharedWorkflow();
            Object.assign(newSharedWorkflow, {
                role: 'workflow:owner',
                user: req.user,
                workflow: savedWorkflow,
            });
            await transactionManager.save(newSharedWorkflow);
        });
        if (!savedWorkflow) {
            this.logger.error('Failed to create workflow', { userId: req.user.id });
            throw new internal_server_error_1.InternalServerError('Failed to save workflow');
        }
        await this.workflowHistoryService.saveVersion(req.user, savedWorkflow, savedWorkflow.id);
        if (tagIds && !config_1.default.getEnv('workflowTagsDisabled') && savedWorkflow.tags) {
            savedWorkflow.tags = this.tagService.sortByRequestOrder(savedWorkflow.tags, {
                requestOrder: tagIds,
            });
        }
        await this.externalHooks.run('workflow.afterCreate', [savedWorkflow]);
        void this.internalHooks.onWorkflowCreated(req.user, newWorkflow, false);
        return savedWorkflow;
    }
    async getAll(req, res) {
        try {
            const roles = this.license.isSharingEnabled()
                ? []
                : ['workflow:owner'];
            const sharedWorkflowIds = await this.workflowSharingService.getSharedWorkflowIds(req.user, roles);
            const { workflows: data, count } = await this.workflowService.getMany(sharedWorkflowIds, req.listQueryOptions);
            res.json({ count, data });
        }
        catch (maybeError) {
            const error = utils.toError(maybeError);
            ResponseHelper.reportError(error);
            ResponseHelper.sendErrorResponse(res, error);
        }
    }
    async getNewName(req) {
        var _a, _b;
        const requestedName = (_a = req.query.name) !== null && _a !== void 0 ? _a : config_1.default.getEnv('workflows.defaultName');
        const name = await this.namingService.getUniqueWorkflowName(requestedName);
        const onboardingFlowEnabled = !config_1.default.getEnv('workflows.onboardingFlowDisabled') &&
            !((_b = req.user.settings) === null || _b === void 0 ? void 0 : _b.isOnboarded) &&
            (await this.userOnboardingService.isBelowThreshold(req.user));
        return { name, onboardingFlowEnabled };
    }
    async getFromUrl(req) {
        if (req.query.url === undefined) {
            throw new bad_request_error_1.BadRequestError('The parameter "url" is missing!');
        }
        if (!/^http[s]?:\/\/.*\.json$/i.exec(req.query.url)) {
            throw new bad_request_error_1.BadRequestError('The parameter "url" is not valid! It does not seem to be a URL pointing to a n8n workflow JSON file.');
        }
        let workflowData;
        try {
            const { data } = await axios_1.default.get(req.query.url);
            workflowData = data;
        }
        catch (error) {
            throw new bad_request_error_1.BadRequestError('The URL does not point to valid JSON file!');
        }
        if ((workflowData === null || workflowData === void 0 ? void 0 : workflowData.nodes) === undefined ||
            !Array.isArray(workflowData.nodes) ||
            workflowData.connections === undefined ||
            typeof workflowData.connections !== 'object' ||
            Array.isArray(workflowData.connections)) {
            throw new bad_request_error_1.BadRequestError('The data in the file does not seem to be a n8n workflow JSON file!');
        }
        return workflowData;
    }
    async getWorkflow(req) {
        var _a;
        const { id: workflowId } = req.params;
        if (this.license.isSharingEnabled()) {
            const relations = ['shared', 'shared.user'];
            if (!config_1.default.getEnv('workflowTagsDisabled')) {
                relations.push('tags');
            }
            const workflow = await this.workflowRepository.get({ id: workflowId }, { relations });
            if (!workflow) {
                throw new not_found_error_1.NotFoundError(`Workflow with ID "${workflowId}" does not exist`);
            }
            const userSharing = (_a = workflow.shared) === null || _a === void 0 ? void 0 : _a.find((shared) => shared.user.id === req.user.id);
            if (!userSharing && !req.user.hasGlobalScope('workflow:read')) {
                throw new unauthorized_error_1.UnauthorizedError('You do not have permission to access this workflow. Ask the owner to share it with you');
            }
            const enterpriseWorkflowService = this.enterpriseWorkflowService;
            enterpriseWorkflowService.addOwnerAndSharings(workflow);
            await enterpriseWorkflowService.addCredentialsToWorkflow(workflow, req.user);
            return workflow;
        }
        const extraRelations = config_1.default.getEnv('workflowTagsDisabled') ? [] : ['workflow.tags'];
        const shared = await this.sharedWorkflowRepository.findSharing(workflowId, req.user, 'workflow:read', { extraRelations });
        if (!shared) {
            this.logger.verbose('User attempted to access a workflow without permissions', {
                workflowId,
                userId: req.user.id,
            });
            throw new not_found_error_1.NotFoundError('Could not load the workflow - you can only access workflows owned by you');
        }
        return shared.workflow;
    }
    async update(req) {
        const { id: workflowId } = req.params;
        const forceSave = req.query.forceSave === 'true';
        let updateData = new WorkflowEntity_1.WorkflowEntity();
        const { tags, ...rest } = req.body;
        Object.assign(updateData, rest);
        const isSharingEnabled = this.license.isSharingEnabled();
        if (isSharingEnabled) {
            updateData = await this.enterpriseWorkflowService.preventTampering(updateData, workflowId, req.user);
        }
        const updatedWorkflow = await this.workflowService.update(req.user, updateData, workflowId, tags, isSharingEnabled ? forceSave : true, isSharingEnabled ? undefined : ['workflow:owner']);
        return updatedWorkflow;
    }
    async delete(req) {
        const { id: workflowId } = req.params;
        const workflow = await this.workflowService.delete(req.user, workflowId);
        if (!workflow) {
            this.logger.verbose('User attempted to delete a workflow without permissions', {
                workflowId,
                userId: req.user.id,
            });
            throw new bad_request_error_1.BadRequestError('Could not delete the workflow - you can only remove workflows owned by you');
        }
        return true;
    }
    async runManually(req) {
        if (this.license.isSharingEnabled()) {
            const workflow = this.workflowRepository.create(req.body.workflowData);
            if (req.body.workflowData.id !== undefined) {
                const safeWorkflow = await this.enterpriseWorkflowService.preventTampering(workflow, workflow.id, req.user);
                req.body.workflowData.nodes = safeWorkflow.nodes;
            }
        }
        return await this.workflowExecutionService.executeManually(req.body, req.user, req.headers['push-ref']);
    }
    async share(req) {
        if (!this.license.isSharingEnabled())
            throw new not_found_error_1.NotFoundError('Route not found');
        const { workflowId } = req.params;
        const { shareWithIds } = req.body;
        if (!Array.isArray(shareWithIds) ||
            !shareWithIds.every((userId) => typeof userId === 'string')) {
            throw new bad_request_error_1.BadRequestError('Bad request');
        }
        const isOwnedRes = await this.enterpriseWorkflowService.isOwned(req.user, workflowId);
        const { ownsWorkflow } = isOwnedRes;
        let { workflow } = isOwnedRes;
        if (!ownsWorkflow || !workflow) {
            workflow = undefined;
            if (req.user.hasGlobalScope('workflow:share')) {
                const sharedRes = await this.sharedWorkflowRepository.getSharing(req.user, workflowId, {
                    allowGlobalScope: true,
                    globalScope: 'workflow:share',
                });
                workflow = sharedRes === null || sharedRes === void 0 ? void 0 : sharedRes.workflow;
            }
            if (!workflow) {
                throw new unauthorized_error_1.UnauthorizedError('Forbidden');
            }
        }
        const ownerIds = (await this.workflowRepository.getSharings(Db.getConnection().createEntityManager(), workflowId, ['shared']))
            .filter((e) => e.role === 'workflow:owner')
            .map((e) => e.userId);
        let newShareeIds = [];
        await Db.transaction(async (trx) => {
            await this.workflowRepository.pruneSharings(trx, workflowId, [...ownerIds, ...shareWithIds]);
            const sharings = await this.workflowRepository.getSharings(trx, workflowId);
            newShareeIds = utils.rightDiff([sharings, (sharing) => sharing.userId], [shareWithIds, (shareeId) => shareeId]);
            if (newShareeIds.length) {
                const users = await this.userRepository.getByIds(trx, newShareeIds);
                await this.sharedWorkflowRepository.share(trx, workflow, users);
            }
        });
        void this.internalHooks.onWorkflowSharingUpdate(workflowId, req.user.id, shareWithIds);
        await this.mailer.notifyWorkflowShared({
            sharer: req.user,
            newShareeIds,
            workflow,
        });
    }
};
exports.WorkflowsController = WorkflowsController;
__decorate([
    (0, decorators_1.Post)('/'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "create", null);
__decorate([
    (0, decorators_1.Get)('/', { middlewares: middlewares_1.listQueryMiddleware }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "getAll", null);
__decorate([
    (0, decorators_1.Get)('/new'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "getNewName", null);
__decorate([
    (0, decorators_1.Get)('/from-url'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "getFromUrl", null);
__decorate([
    (0, decorators_1.Get)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "getWorkflow", null);
__decorate([
    (0, decorators_1.Patch)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "update", null);
__decorate([
    (0, decorators_1.Delete)('/:id'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "delete", null);
__decorate([
    (0, decorators_1.Post)('/run'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "runManually", null);
__decorate([
    (0, decorators_1.Put)('/:workflowId/share'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], WorkflowsController.prototype, "share", null);
exports.WorkflowsController = WorkflowsController = __decorate([
    (0, decorators_1.RestController)('/workflows'),
    __metadata("design:paramtypes", [Logger_1.Logger,
        InternalHooks_1.InternalHooks,
        ExternalHooks_1.ExternalHooks,
        tag_repository_1.TagRepository,
        workflow_service_ee_1.EnterpriseWorkflowService,
        workflowHistory_service_ee_1.WorkflowHistoryService,
        tag_service_1.TagService,
        naming_service_1.NamingService,
        userOnboarding_service_1.UserOnboardingService,
        workflow_repository_1.WorkflowRepository,
        workflow_service_1.WorkflowService,
        workflowExecution_service_1.WorkflowExecutionService,
        workflowSharing_service_1.WorkflowSharingService,
        sharedWorkflow_repository_1.SharedWorkflowRepository,
        user_repository_1.UserRepository,
        License_1.License,
        email_1.UserManagementMailer,
        credentials_service_1.CredentialsService])
], WorkflowsController);
//# sourceMappingURL=workflows.controller.js.map